package org.netkernel.ext.system.accessor;


import org.netkernel.container.IMessages;
import org.netkernel.layer0.bnf.GrammarParseException;
import org.netkernel.layer0.meta.GrammarException;
import org.netkernel.layer0.meta.IEndpointMeta;
import org.netkernel.layer0.meta.IIdentifierGrammar;
import org.netkernel.layer0.meta.impl.EndpointMetaBuilder;
import org.netkernel.layer0.meta.impl.MetadataUtils;
import org.netkernel.layer0.nkf.INKFRequestContext;
import org.netkernel.layer0.representation.IHDSNode;
import org.netkernel.layer0.representation.impl.HDSBuilder;
import org.netkernel.layer0.util.OrderedCheapMultiStringMap;
import org.netkernel.layer0.util.PairList;
import org.netkernel.layer0.util.XMLUtils;
import org.netkernel.module.standard.endpoint.StandardAccessorImpl;
import org.w3c.dom.Document;
import org.w3c.dom.Node;


public class GrammarRuntimeEndpoint extends StandardAccessorImpl
{
	public GrammarRuntimeEndpoint()
	{	this.declareThreadSafe();
	}

	@Override
	public void onSource(INKFRequestContext aContext) throws Exception
	{
		IMessages messages=aContext.getKernelContext().getKernel().getLogger();
		IIdentifierGrammar grammar = null;
		try
		{	Document grammarDoc=(Document)aContext.source("arg:grammar", Document.class);
			EndpointMetaBuilder epm=new EndpointMetaBuilder();
			MetadataUtils.parseGrammar(grammarDoc.getDocumentElement(), epm, messages);
			IEndpointMeta meta=epm.getMeta(messages);
			grammar=meta.getIdentifierGrammar();
		}
		catch (GrammarParseException e)
		{	IHDSNode errors=onParseErrors(e);
			aContext.createResponseFrom(errors);
		}
		
		if (grammar!=null)
		{
			if (aContext.getThisRequest().argumentExists("identifier"))
			{
				String identifier=aContext.source("arg:identifier", String.class);
				IHDSNode parsed=onParse(grammar,identifier);
				aContext.createResponseFrom(parsed);
			}
			else if (aContext.getThisRequest().argumentExists("parts"))
			{
				OrderedCheapMultiStringMap parts = aContext.source("arg:parts", OrderedCheapMultiStringMap.class);
				IHDSNode constructed=onConstruct(grammar,parts,messages);
				aContext.createResponseFrom(constructed);
			}
		}
	}
	
	private IHDSNode onParse(IIdentifierGrammar aGrammar, String aIdentifier) throws GrammarException
	{
		HDSBuilder builder=new HDSBuilder();
		builder.pushNode("results",null);
			builder.addNode("matches",aGrammar.matches(aIdentifier));
			builder.pushNode("parts",null);
				OrderedCheapMultiStringMap map=aGrammar.parse(aIdentifier);
				for(int i=0; i<map.size(); i++)
				{	builder.pushNode("part",null);
						builder.addNode("name",map.getKey(i));
						builder.addNode("value",map.getValue(i));
					builder.popNode();
				}
			builder.popNode();
		builder.popNode();
		return builder.getRoot();
	}
	
	private IHDSNode onConstruct(IIdentifierGrammar aGrammar, OrderedCheapMultiStringMap aParts, IMessages aMessages) throws GrammarException
	{
		HDSBuilder builder=new HDSBuilder();
		builder.pushNode("results",null);
		builder.pushNode("parts");
		for(int i=0; i<aParts.size(); i++)
		{	builder.pushNode("part",null);
				builder.addNode("name",aParts.getKey(i));
				builder.addNode("value",aParts.getValue(i));
			builder.popNode();
		}
		builder.popNode();
		try
		{	String identifier=aGrammar.construct(aParts, aMessages);
			builder.addNode("constructed", identifier);			
		}
		catch (GrammarException e)
		{	builder.addNode("construct-error",e.getMessage());
		}
		builder.popNode();
		return builder.getRoot();		
	}
	
	
	private IHDSNode onParseErrors(GrammarParseException aException)
	{
		HDSBuilder builder=new HDSBuilder();
		builder.pushNode("results",null);
		builder.pushNode("errors",null);
		PairList pl=aException.getValidationErrors();
		for (int i=0; i<pl.size(); i++)
		{
			builder.pushNode("error",null);
			Node loc=(Node)pl.getValue1(i);
			builder.addNode("loc",XMLUtils.getPathFor(loc));
			String msg=(String)pl.getValue2(i);
			builder.addNode("msg",msg);
			builder.popNode();
		}
		builder.popNode();
		builder.popNode();
		return builder.getRoot();
	}
}