package org.netkernel.ext.system.accessor;

import org.netkernel.layer0.boot.BootPersistence;
import org.netkernel.layer0.logging.LogManager;
import org.netkernel.layer0.nkf.INKFLocale;
import org.netkernel.layer0.nkf.INKFRequestContext;
import org.netkernel.layer0.nkf.INKFResponse;
import org.netkernel.layer0.representation.ByteArrayRepresentation;
import org.netkernel.layer0.representation.IHDSNode;
import org.netkernel.layer0.representation.IReadableBinaryStreamRepresentation;
import org.netkernel.layer0.representation.impl.HDSFactory;
import org.netkernel.layer0.util.XMLUtils;
import org.netkernel.module.standard.endpoint.StandardAccessorImpl;
import org.netkernel.request.IRequestResponseFields;
import org.netkernel.request.impl.RequestResponseFieldsImpl;
import org.netkernel.util.Utils;
import org.w3c.dom.Document;

public class LogLevelAccessor extends StandardAccessorImpl
{
	private LogManager mLogManager;
	public static final String LOG_LEVELS_PERSISTENCE_FILE="logLevels.xml";
	
	public void postCommission(INKFRequestContext aContext) throws Exception
	{	
		mLogManager=LogManager.getInstance();
		try
		{	IReadableBinaryStreamRepresentation persistedState=BootPersistence.sourceBootResource(LOG_LEVELS_PERSISTENCE_FILE, getKernel());
			IHDSNode parsed=HDSFactory.parseXML(persistedState.getInputStream());
			mLogManager.setLevels(parsed);
		}
		catch (Exception e)
		{	// no persistence found
			System.out.println(Utils.throwableToString(e));
		}
	}
	
	public void preDecommission(INKFRequestContext aContext) throws Exception
	{	
	}
	
	public void onSource(INKFRequestContext aContext) throws Exception
	{	INKFResponse resp=aContext.createResponseFrom(mLogManager.getState());
		resp.setExpiry(INKFResponse.EXPIRY_ALWAYS);
	}
	
	public void onSink(INKFRequestContext aContext) throws Exception
	{	IHDSNode state=aContext.sourcePrimary(IHDSNode.class);
		mLogManager.setLevels(state);
		try
		{	saveState();
		}
		catch (Exception e)
		{	aContext.logFormatted(INKFLocale.LEVEL_WARNING, "MSG_FAILED_TO_SAVE_LOG_LEVELS", Utils.throwableToString(e));
		}
	}
	
	public IRequestResponseFields getState()
	{	RequestResponseFieldsImpl state=new RequestResponseFieldsImpl(super.getState());
		state.put("levels", mLogManager.getState());
		return state;
	}
	
	private void saveState() throws Exception
	{	IHDSNode state=mLogManager.getState();
		Document d=HDSFactory.toDOM(state,false);
		String s=XMLUtils.toXML(d, true, false);
		ByteArrayRepresentation rep=new ByteArrayRepresentation(s.getBytes("UTF-8"));
		BootPersistence.sinkBootResource(LOG_LEVELS_PERSISTENCE_FILE, rep, getKernel());
	}
}
