package org.netkernel.ext.system.representation;

import org.netkernel.layer0.representation.IHDSNode;
import org.netkernel.urii.IMetaRepresentation;

public interface IRepDeployedModules
{
	/** Determine the hash string for a module, space or physical endpoint located from
	 * the module manager root. Returns null if component not found. 
	 * @param aComponent either an IModule, ISpace, IEndpoint
	 * @return a hash string
	 */
	String hashForComponent(Object aComponent);
	
	/** Locate the module, space or physical endpoint for a given hash string generated
	 * by hashForComponent(). Returns null if component not found.
	 * @param aHash a hash string
	 * @return the located component reference
	 */
	Object componentForHash(String aHash);
	
	/** Locate the metadata for the given component
	 * @param aHash
	 * @return
	 */
	IMetaRepresentation metadataForHash(String aHash);
	
	/** Determine the hash string for a static module or space or part located from
	 * the module manager root. Returns null if meta not found. 
	 * @param aMeta
	 * @return a hash string
	 */
	String hashForMetadata(IMetaRepresentation aMeta);
	
	/** Locate hash for prototype or representation of a particular class
	 * @param aClass
	 * @return
	 */
	String hashForClass(Class aClass);
	
	/** Locate hash for a space element with given id
	 * @param aClass
	 * @return
	 */
	String hashForSpaceElement(String aSpaceElementId);
	
	/** Return a hierarchical representation of the structure of deployment
	 * modules->spaces->physical-endpoints->space-elements
	 * @return
	 */
	IHDSNode getHierarchy();
}
