package org.netkernel.ext.system.transrepresentation;

import java.util.List;
import org.netkernel.ext.system.representation.IRepDeployedModules;
import org.netkernel.layer0.nkf.INKFRequestContext;
import org.netkernel.layer0.representation.IHDSNode;
import org.netkernel.layer0.representation.impl.HDSBuilder;
import org.netkernel.layer0.representation.impl.HDSFactory;
import org.netkernel.module.standard.endpoint.StandardTransreptorImpl;
import org.netkernel.scheduler.TraceDebugCapture;
import org.netkernel.scheduler.TraceDebugCapture.TraceEntry;
import org.netkernel.urii.IEndpoint;
import org.netkernel.urii.ISpace;
import org.w3c.dom.Document;

public class ResolutionDebugSerializer extends StandardTransreptorImpl
{
	public ResolutionDebugSerializer()
	{
		this.declareThreadSafe();
		this.declareFromRepresentation(TraceDebugCapture.class);
		this.declareToRepresentation(IHDSNode.class);
		this.declareToRepresentation(Document.class);
	}
	
	public void onTransrept(INKFRequestContext aContext) throws Exception
	{
		
		TraceDebugCapture tdc=aContext.sourcePrimary(TraceDebugCapture.class);
		IRepDeployedModules modules=aContext.source("active:moduleList", IRepDeployedModules.class);
		IHDSNode hds=serializeResolutionDebug(tdc, modules);
		Class to=aContext.getThisRequest().getRepresentationClass();
		if (to==IHDSNode.class)
		{	aContext.createResponseFrom(hds);
		}
		else
		{	Document d=HDSFactory.toDOM(hds);
			if (to==Document.class)
			{	aContext.createResponseFrom(d);
			}
		}
	}
	
	public static IHDSNode serializeResolutionDebug(TraceDebugCapture aTDC,  IRepDeployedModules aDMR) 
	{
		HDSBuilder b=new HDSBuilder();
		b.pushNode("trace");
		//List<TraceDebugCapture> hierarchy=new ArrayList<TraceDebugCapture>();
		int j=0;
		while (j<aTDC.getEntries().size())
		{	j=recurseWriteTDC(b,aTDC,aDMR,j,null);
		}	
		return b.getRoot();		
	}
	
	private static int recurseWriteTDC(HDSBuilder b, TraceDebugCapture aTDC, IRepDeployedModules aDMR, int aIndex, String aSpaceHash)
	{	List<TraceEntry> entries=aTDC.getEntries();
		//aHierarchy.add(aTDC);
		int count=entries.size();
		TraceEntry te=entries.get(aIndex);
		int depth=te.getDepth();
		b.pushNode("n",te.getMessage());
		Object e=te.getEntity();
		String hash=null;
		String spaceHash=aSpaceHash;
		String type=null;
		if(e!=null)
		{	if (e instanceof ISpace)
			{	hash=spaceHash=aDMR.hashForComponent(e);
				type="space";
				//System.out.println("SPACE: "+hash+" "+te.getMessage());
			}
			else if (e instanceof IEndpoint)
			{	hash=aDMR.hashForComponent(e);
				if (hash!=null)
				{	int i=hash.lastIndexOf('/');
					spaceHash=hash.substring(0,i);
					type="physical";
				}
			}
			else if (e instanceof String)
			{	
				hash=aSpaceHash+"/"+(String)e;
				type="logical";
				//System.out.println("LOGICAL: "+hash+" "+te.getMessage());
			}
			/*
			b.pushNode("entity", null);
			if(e instanceof NKFResolutionDebugEntity)
			{	
				NKFResolutionDebugEntity rde=(NKFResolutionDebugEntity)e;
				b.addNode("endpoint", rde.getEndpoint());
				b.addNode("scope", rde.getScope());
				b.addNode("hash", aDMR.hashForComponent(rde.getEndpoint()));
			}
			b.popNode();
				*/
		}
		if (hash!=null)
		{	b.pushNode("entity",hash);
			b.addNode("@type",type);
			b.popNode();
		}
		
		
		int j=aIndex+1;
		while (j<count && entries.get(j).getDepth()>depth)
		{	j=recurseWriteTDC(b,aTDC,aDMR,j,spaceHash);
		}
		b.popNode();
		//aHierarchy.remove(aHierarchy.size()-1);
		return j;
	}


}
